from __future__ import annotations
from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from .person import Person
    from .workspace import Workspace
import logging

import wxcadm
from .common import *
from typing import Optional, Union
from .exceptions import *


class Device:
    def __init__(self, parent: Person | Workspace, config: dict):
        self.parent = parent
        """ The :py:class:`Org`, :py:class:`Person` or :py:class:`Workspace` that created this instance """
        self.id: str = config['id']
        """ The Device ID """
        self.tags: list = config.get('tags', config.get('description', ''))
        """ The device tags """
        self.model: str = config.get('model', config.get('product', ''))
        """ The model name of the device """
        self.mac: str = config.get('mac')
        """ The device MAC address """
        self.is_owner: bool = config.get('primaryOwner', None)
        """ Whether the Person or Workspace is the primary owner of the device """
        self.activation_state: str = config.get('activationState', None)
        """ The device activation state """
        self.type: str = config['type']
        """ The type of device association """
        self.ip_address: Optional[str] = config.get('ipAddress', config.get('ip', ''))
        """ The IP Address of the device """
        self._settings: Optional[dict] = None
        self.display_name = config.get('displayName', None)
        """ The name of device as displayed in Webex """
        self.capabilities: list = config.get('capabilities', [])
        """ The capabilities of the device """
        self.user_permissions: list = config.get('permissions', [])
        """ The permissions the user has for this device. For example, "xapi" means this user is allowed to use xapi """
        self.connection_status: Optional[str] = config.get('connectionStatus', None)
        """ Whether the device is connected or not """
        self.serial: Optional[str] = config.get('serial', None)
        """ The serial number of the device, if available """
        self.software: Optional[str] = config.get('software', None)
        """ The operating system name data and version tag, if available """
        self.upgrade_channel: Optional[str] = config.get('upgradeChannel', None)
        """ The upgrade channel the device is assigned to """
        self.created: Optional[str] = config.get('created', None)
        """ The date and time that the device was created on Webex """
        self.first_seen: Optional[str] = config.get('firstSeen', None)
        """ The date and time that the device was first seen online """
        self.last_seen: Optional[str] = config.get('lastSeen', None)
        """ The date and time that the device was last seen online """
        self.owner = None
        """ The :py:class:`Person` or :py:class:`Workspace` that owns the device primarily """

        if 'personId' in config.keys() and isinstance(self.parent, wxcadm.Org):
            self.owner = self.parent.get_person_by_id(config['personId'])
        elif 'workspaceId' in config.keys() and isinstance(self.parent, wxcadm.Org):
            self.owner = self.parent.workspaces.get_by_id(config['workspaceId'])

    def change_tags(self, operation: str, tag: Optional[Union[str, list]] = None):
        """ Add a tag to the list of tags for this device

        It is also possible to change the tags by modifying the ``settings`` directly, but this method ensures that
        no other settings are touched.

        Args:
            operation (str): Valid values are:
                "add" - Add a tag or list of tags to the existing tags
                "remove" - Remove all tags for the device
                "replace" - Replace all tags with the provided tag or list of tags
            tag (str, list, optional): The tag value or list of values. Valid for "add" and "replace" operations.

        Returns:

        """
        # Value checking
        if (operation.lower() in ['add', 'replace']) and tag is None:
            raise ValueError("add or replace operations require a 'tag' param")
        if isinstance(tag, str):
            tag = [tag]

        payload = {
            "op": operation.lower(),
            "path": "tags",
            "value": tag
        }
        response = webex_api_call("patch", f"/v1/devices/{self.id}", payload=payload)

        return True

    @property
    def settings(self):
        """ The device settings.

        The available settings vary by device type, so the raw dictionary from Webex is returned.

        """
        if self._settings is None:
            response = webex_api_call('get', f'/v1/telephony/config/devices/{self.id}/settings',
                                      params={'model': self.model})
            self._settings = response
        return self._settings

    @settings.setter
    def settings(self, config: dict):
        try:
            webex_api_call('put', f'/v1/telephony/config/devices/{self.id}/settings',
                                  payload=config)
        except APIError:
            logging.warning("The API call to set the device settings failed")

        self._settings = config

    def apply_changes(self) -> bool:
        """ Issues request to the device to download and apply changes to the configuration.

        Returns:
            bool: True on success, False otherwise

        """
        try:
            webex_api_call('post', f'/v1/telephony/config/devices/{self.id}/actions/applyChanges/invoke')
        except APIError:
            return False

        return True

    def delete(self):
        """ Delete the device from Webex Calling and from Control Hub completely

        Returns:
            bool: True on success. An exception will be thrown otherwise

        """
        response = webex_api_call("delete", f"/v1/devices/{self.id}")

        return True




from __future__ import annotations
from typing import Optional
import requests
from .common import *
import wxcadm
from wxcadm import log
from datetime import datetime, timedelta


class Reports:
    """ The Reports class provides an interface to all reports available via the Webex API """
    def __init__(self, org: wxcadm.Org):
        self.org = org
        self._templates: Optional[list] = None
        self.created_reports: list = []
        """ A list of IDs for all reports created during this session """

    @property
    def templates(self):
        """ The Report Templates available

        Returns:
            dict: A dict of Report Templates

        """
        log.info('Getting reports templates')
        self._templates = webex_api_call('get', '/v1/report/templates')
        return self._templates

    def list_reports(self):
        """ List the reports available to the authorized user via the Webex API

        .. note::
            Webex isolates reports created via the API from reports created within Control Hub. The reports listed
            by this method will show only the reports created using the API.

        Returns:
            list[dict]: A list of all reports as dicts

        """
        log.info('Getting list of reports')
        my_reports = webex_api_call('get', '/v1/reports')
        return my_reports

    def create_report(self, template_id: int,
                      start_date: Optional[str] = None,
                      end_date: Optional[str] = None,
                      site_list: Optional[list] = None):
        """ Create a new report

        The arguments vary based on the Template used. By looking at the `templates` list, you can see the requirements
        in the `validations` key.

        Args:
            template_id (int): The Template ID to use for the report
            start_date (str): The first date to include in the report
            end_date (str): The last date to include in the report
            site_list (list[str], optional): A list of Webex meeting sites

        Returns:
            str: The Report ID that was created

        """
        log.info(f'Creating report with Template ID: {template_id}')
        log.debug(f'\tStart: {start_date}, End: {end_date}')
        log.debug(f'\tSite List: {site_list}')
        payload = {}
        payload['templateId'] = template_id
        if start_date is not None:
            payload['startDate'] = start_date
        if end_date is not None:
            payload['endDate'] = end_date
        if site_list is not None:
            payload['siteList'] = site_list

        response = webex_api_call('post', '/v1/reports', payload=payload)
        log.debug(f'API response: {response}')
        report_id = response['items']['Id']
        log.info(f'Report ID: {report_id}')
        self.created_reports.append(report_id)
        return report_id

    @staticmethod
    def report_status(report_id: str):
        """ The text status of the provided report_id

        Args:
            report_id (str): The Report ID

        Returns:
            str: The status of the report

        """
        log.info(f"Getting report status for Report ID {report_id}")
        response = webex_api_call('get', f'/v1/reports/{report_id}')
        status = response[0]['status']
        return status

    def get_report_lines(self, report_id: str):
        """ Download and return the lines from a report

        This method returns the full report in a list, with each line as an entry.

        Args:
            report_id (str): The Report ID

        Returns:
            list: The report lines

        """
        log.info(f'Getting report with ID {report_id}')
        response = webex_api_call('get', f'/v1/reports/{report_id}')
        url = response[0]['downloadURL']
        log.debug(f'\tDownload URL: {url}')
        r = requests.get(url, headers=self.org._headers)
        if r.ok:
            report_lines = r.text.replace('\xef\xbb\xbf', '')
            return report_lines.split('\n')
        else:
            return False

    def cdr_report(self, start: Optional[str] = None, end: Optional[str] = None, days: Optional[int] = None):
        """ Create a CDR report

        This method serves as a shortcut to :py:meth:`create_report()` for a Calling Detailed Call History report. It
        finds the correct template and creates the report using the given start and end dates, or the ``days``
        argument can be provided to report the last X days.

        Args:
            start (str, optional): The first date to include in the report (YYYY-MM-DD)
            end (str, optional): The last date to include in the report (YYYY-MM-DD)
            days (int, optional): The number of days to report on, ending on yesterday's date

        Returns:
            str: The Report ID

        """
        if start is None and days is None:
            raise ValueError("You must provide either a start date or the number of days")
        if start is not None and days is not None:
            raise ValueError("You must not specify a start date and number of days")
        if days is not None:
            yesterday = datetime.now() - timedelta(1)
            end = datetime.strftime(yesterday, "%Y-%m-%d")
            start_date = yesterday - timedelta(days)
            start = datetime.strftime(start_date, "%Y-%m-%d")

        # Find the CDR Report Template
        for template in self.templates:
            if template['title'] == 'Calling Detailed Call History':
                cdr_template = template['Id']
                break

        report_id = self.create_report(template_id=cdr_template,
                                       start_date=start,
                                       end_date=end)
        return report_id



from __future__ import annotations

from typing import Optional
from collections import UserList

import wxcadm.org
from wxcadm import log
from .common import *
from .location import Location


class NumberManagementJob:
    """ The class for Number Moves.

    This class is never instantiated directly. It is created with the :class:`NumberMoveJobList`

    """
    def __init__(self, parent: wxcadm.Org, id: str, details: Optional[dict] = None):
        self.parent = parent
        self.id = id
        """ The Number Move Job ID """
        self.details: dict
        """ The full details of the Number Move Job from Webex """
        if details is None:
            self.details = self._get_details()
        else:
            self.details = details

    def _get_details(self):
        response = webex_api_call('get', f'v1/telephony/config/jobs/numbers/manageNumbers/{self.id}')
        return response

    @property
    def completed(self) -> bool:
        """ Whether the job has completed execution

        Note that this does not indicate that the job was successful. The :attr:`success` attribute should be used to
        determine whether the job was successful or not

        """
        self._get_details()
        if self.details['latestExecutionStatus'] == 'COMPLETED':
            return True
        else:
            return False

    @property
    def success(self) -> bool:
        """ Whether the job was successful

        This property will return False even if the job is not complete. It should really be called only after
        :attr:`completed` is True

        """
        self._get_details()
        if int(self.details['counts']['numbersFailed']) > 0:
            return False

        total_numbers = int(self.details['counts']['totalNumbers'])
        success_total = int(self.details['counts']['numbersDeleted']) + int(self.details['counts']['numbersMoved'])

        if success_total == total_numbers:
            return True
        else:
            return False


class NumberManagementJobList(UserList):
    _endpoint = "v1/telephony/config/jobs/numbers/manageNumbers"
    _endpoint_items_key = "items"
    _item_endpoint = "v1/telephony/config/jobs/numbers/manageNumbers/{item_id}"
    _item_class = NumberManagementJob

    def __init__(self, parent: wxcadm.Org):
        super().__init__()
        log.debug("Initializing NumberMoveJobList")
        self.parent: wxcadm.Org = parent
        self.data: list = self._get_data()

    def _get_data(self) -> list:
        log.debug("_get_data() started")
        params = {}

        if isinstance(self.parent, wxcadm.Org):
            log.debug(f"Using Org ID {self.parent.id} as Data filter")
            params['orgId'] = self.parent.id
            params['max'] = 1000
        else:
            log.warn("Parent class is not Org so all items will be returned")
        response = webex_api_call('get', self._endpoint, params=params)
        log.info(f"Found {len(response)} items")

        items = []
        for entry in response:
            items.append(self._item_class(parent=self.parent, id=entry['id'], details=entry))
        return items

    def refresh(self):
        """ Refresh the list of instances from Webex

        Returns:
            bool: True on success, False otherwise.

        """
        self.data = self._get_data()
        return True

    def get(self, id: str):
        """ Get the instance associated with a given Job ID

        Args:
            id (str, optional): The Number Management Job ID to find

        Returns:
            NumberManagementJob: The Number Management Job instance correlating with the given ID

        """
        for item in self.data:
            if item.id == id:
                return item
        return None

    def create(self,
               job_type: str,
               target_location: wxcadm.Location | str,
               numbers: list
               ):
        """ Create a Number Management Job

        .. note::

            At this time, only the "move" value is accepted for the `job_type` argument, because Webex only supports
            move operations via the API. Also, the API currently limits a Number Move to a single number per job. The
            API schema is written to support more, so it may be supported at some point, but for now, the `numbers` list
            must contain only a single entry.

        Args:
            job_type (str): Only "move" is supported currently
            target_location (Location | str): A :class:`Location` instance or a string representing the Location ID to
            move the numbers to
            numbers (list[str]): A list of numbers, as strings, to move to the target Location

        Returns:
            NumberManagementJob: The created NumberManagementJob

        """
        # Determine if the numbers are valid
        numbers_to_move = []
        # Get a copy of the Org numbers for processing
        org_numbers = self.parent.numbers
        for number in numbers:
            number_found = False
            for org_number in org_numbers:
                if number in org_number['phoneNumber']:
                    number_found = True
                    numbers_to_move.append(org_number)
            if number_found is False:
                raise KeyError(f"{number} was not found in the Org")

        if isinstance(target_location, wxcadm.Location):
            target_location_id = target_location.id
        else:
            target_location_id = target_location

        # Build the payload for the move job
        payload = {'operation': 'MOVE', 'targetLocationId': target_location_id, 'numberList': []}
        for number in numbers_to_move:
            payload['numberList'].append({'locationId': number['location'].id, 'numbers': [number['phoneNumber']]})

        response = wxcadm.webex_api_call('post', 'v1/telephony/config/jobs/numbers/manageNumbers',
                                         params={'orgId': self.parent.id},
                                         payload=payload)
        job_id = response['id']
        self.refresh()
        return NumberManagementJob(parent=self.parent, id=job_id)



from __future__ import annotations

from typing import Optional
from collections import UserList

import wxcadm
from wxcadm import log
from .location_features import LocationSchedule, CallParkExtension
from .call_queue import CallQueueList
from .hunt_group import HuntGroupList
from .auto_attendant import AutoAttendantList
from .pickup_group import PickupGroupList
from .common import *
from .exceptions import APIError


class LocationList(UserList):
    def __init__(self, parent: wxcadm.Org):
        super().__init__()
        log.debug("Initializing LocationList instance")
        self.parent: wxcadm.Org = parent
        self.data: list = self._get_items()

    def refresh(self):
        """ Refresh the list of Locations from Webex """
        self.data = self._get_items()

    def _get_items(self):
        if isinstance(self.parent, wxcadm.Org):
            log.debug("Using Org ID as data filter")
            params = {'orgId': self.parent.id}
        else:
            raise ValueError("Unsupported parent class")

        log.debug("Getting Location list")
        response = webex_api_call('get', f'v1/locations', params=params)
        log.debug(f"Received {len(response)} entries")
        items = []
        for entry in response:
            items.append(Location(parent=self.parent,
                                  location_id=entry['id'],
                                  name=entry['name'],
                                  address=entry['address'],
                                  time_zone=entry['timeZone'],
                                  preferred_language=entry['preferredLanguage']))
        return items

    def get(self, id: str = None, name: str = None, spark_id: str = None):
        """ Get the Location instance associated with a given ID, Name, or Spark ID

        Only one parameter should be supplied in normal cases. If multiple arguments are provided, the Locations will be
        searched in order by ID, Name, and finally Spark ID. If no arguments are provided, the method will raise an
        Exception.

        Args:
            id (str, optional): The Location ID to find
            name (str, optional): The Location Name to find
            spark_id (str, optional): The Spark ID to find

        Returns:
            Location: The Location instance correlating to the given search argument.

        Raises:
            ValueError: Raised when the method is called with no arguments

        """
        if id is None and name is None and spark_id is None:
            raise ValueError("A search argument must be provided")
        if id is not None:
            for location in self.data:
                if location.id == id:
                    return location
        if name is not None:
            for location in self.data:
                if location.name == name:
                    return location
        if spark_id is not None:
            for location in self.data:
                if location.spark_id == spark_id:
                    return location
        return None

    def create(self,
               name: str,
               time_zone: str,
               preferred_language: str,
               address: dict):
        """ Create a new Location

        Args:
            name (str): The name of the Location
            time_zone (str): The Time Zone of the Location (e.g. America/Chicago)
            preferred_language (str): The preferred language for phones and menus (i.e. en_US)
            address (dict): A dictionary containing the address elements. The format of this dict should be::

                {
                    "address1": "100 N. Main",
                    "address2": "Suite 200",
                    "city": "Houston",
                    "state": "TX",
                    "postalCode": "32123",
                    "country": "US"
                }

        Returns:
            dict: The response from Webex with the Location details.

        """
        payload = {
            'name': name,
            'timeZone': time_zone,
            'preferredLanguage': preferred_language,
            'announcementLanguage': None,
            'address': address
        }
        response = webex_api_call('post', 'v1/locations', payload=payload)
        self.refresh()
        return self.get(id=response['id'])

    def webex_calling(self, enabled: bool = True) -> list[Location]:
        """ Return a list of :py:class:`Location` where Webex Calling is enabled/disabled

        Args:
            enabled (bool, optional): True (default) returns Webex Calling people. False returns Locations without
                Webex Calling

        Returns:
            list[:py:class:`Location`]: List of :py:class:`Location` instances. An empty list is returned if none match
            the given criteria

        """
        locations = []
        entry: Location
        for entry in self.data:
            if entry.calling_enabled is enabled:
                locations.append(entry)
        return locations


class Location:
    def __init__(self, parent: wxcadm.Org, location_id: str,
                 name: str,
                 time_zone: str,
                 preferred_language: str,
                 announcement_language: str = None,
                 address: dict = None):
        """Initialize a Location instance

        .. note::
            Location instances are normally not instantiated manually and are done automatically with the
            :meth:`Org.get_locations` method.

        Args:
            location_id (str): The Webex ID of the Location
            name (str): The name of the Location
            time_zone (str): The time zone of the Location
            preferred_language (str): The preferred language at the Location
            announcement_language (str): The language for audio announcements at the Location
            address (dict): The address information for the Location

        Returns:
             Location (object): The Location instance

        """
        self._parent = parent
        self.parent = parent
        self._headers = parent._headers
        self.id: str = location_id
        """The Webex ID of the Location"""
        self.name: str = name
        """The name of the Location"""
        self.address: dict = address
        """The address of the Location"""
        self.time_zone: str = time_zone
        """ The Location time zone"""
        self.preferred_language: str = preferred_language
        """ The preferred language at the Location"""
        self.announcement_language: str = announcement_language
        """ The language for audio announcements at the Location"""
        self.calling_enabled: bool = False
        """ Whether or not the Location is enabled for Webex Calling """
        self.calling_config: Optional[dict] = None
        """ The Webex Calling config for the Location, if enabled """
        self._pickup_groups: Optional[PickupGroupList] = None
        self._call_queues: Optional[CallQueueList] = None
        self._hunt_groups: Optional[HuntGroupList] = None

        # Get the Webex Calling config and determine if the Location is Calling-enabled
        self._get_calling_config()

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.id

    @property
    def workspace_location(self):
        """ Get the :class:`WorkspaceLocation` associated with this Location

        Returns:
            WorkspaceLocation: The WorkspaceLocation associated with this Location

        """
        return self.parent.workspace_locations.get(name=self.name)


    def _get_calling_config(self):
        try:
            response = webex_api_call("get", f"v1/telephony/config/locations/{self.id}")
            if 'id' in response.keys():
                self.calling_enabled = True
                self.calling_config = response
        except APIError:
            return None

    @property
    def org_id(self):
        """ The Org ID to which the Location belongs. :attr:`parent.id` can also be used. """
        return self.parent.id

    def enable_webex_calling(self) -> bool:
        """ Enable the Location for Webex Calling

        If the Location is already a Webex Calling Location, no action will be taken and the method will return True.

        Returns:
            bool: True on success

        """
        if self.calling_enabled is False:
            log.info(f"Enabling Webex Calling for Location {self.name}")
            payload = {
                "id": self.id,
                "name": self.name,
                "timeZone": self.time_zone,
                "preferredLanguage": self.preferred_language.lower(),
                "announcementLanguage": self.preferred_language.lower(),
                "address": self.address
            }
            webex_api_call("post", "v1/telephony/config/locations", payload=payload)
            self._get_calling_config()
        return True

    def delete(self):
        """ Delete a Location

        .. warning::

            There is currently no way to delete a Location outside of Control Hub. The method is defined here so users
            aren't looking for it expecting it to be there. It will always return False. When the API is exposed,
            this method will be updated to support it.

        Returns:
            bool: Always returns False and no action is taken on Webex

        """
        return False

    @property
    def spark_id(self):
        """The ID used by all underlying services."""
        return decode_spark_id(self.id)

    @property
    def hunt_groups(self):
        """ :class:`HuntGroupList` list of :class:`HuntGroup` instances for this Location """
        log.info(f"Getting Hunt Groups for Location: {self.name}")
        if self.calling_enabled is False:
            log.debug("Not a Webex Calling Location")
            return None
        self._hunt_groups = HuntGroupList(self)
        return self._hunt_groups

    @property
    def announcements(self):
        """ List of :py:class:`Announcement` instances for the Location

        .. note::
            This does not include announcements built at the Organization level

        """
        annc_list = self._parent.announcements.get_by_location_id(self.id)
        return annc_list

    @property
    def auto_attendants(self):
        """ List of AutoAttendant instances for this Location """
        log.info(f"Getting Auto Attendants for Location: {self.name}")
        return AutoAttendantList(self)

    @property
    def call_queues(self) -> Optional[CallQueueList]:
        """ :class:`CallQueueList` of :class:`CallQueue` instances for this Location """
        if self._call_queues is None:
            log.info(f"Getting Call Queues for Location: {self.name}")
            if self.calling_enabled is False:
                log.warn("Not a Webex Calling Location")
                return None
            self._call_queues = CallQueueList(self)
        return self._call_queues

    @property
    def numbers(self):
        """ All the Numbers for the Location

        Returns:
            list[dict]: List of dict containing information about each number

        """
        log.info("Getting Location numbers from Webex")
        if self.calling_enabled is False:
            log.debug("Not a Webex Calling Location")
            return None
        params = {'locationId': self.id}
        response = webex_api_call("get", "v1/telephony/config/numbers", params=params)
        loc_numbers = response['phoneNumbers']
        for num in loc_numbers:
            if "owner" in num:
                if "id" in num['owner']:
                    person = self._parent.get_person_by_id(num['owner']['id'])
                    if person is not None:
                        num['owner'] = person
                    else:
                        if num['owner']['type'].upper() == "HUNT_GROUP":
                            hunt_group = self._parent.get_hunt_group_by_id(num['owner']['id'])
                            if hunt_group is not None:
                                num['owner'] = hunt_group
                        elif num['owner']['type'].upper() == "GROUP_PAGING":
                            paging_group = self._parent.get_paging_group(id=num['owner']['id'])
                            if paging_group is not None:
                                num['owner'] = paging_group
                        elif num['owner']['type'].upper() == "CALL_CENTER":
                            call_queue = self._parent.get_call_queue_by_id(num['owner']['id'])
                            if call_queue is not None:
                                num['owner'] = call_queue
                        elif num['owner']['type'].upper() == "AUTO_ATTENDANT":
                            auto_attendant = self._parent.auto_attendants.get(id=num['owner']['id'])
                            if auto_attendant is not None:
                                num['owner'] = auto_attendant
        return loc_numbers

    @property
    def available_numbers(self):
        """Returns all available numbers for the Location.

        Returns both Active and Inactive numbers so that numbers can be assigned prior to activation/porting/

        Returns:
            list[dict]: A list of available numbers, in dict form

        """
        log.debug('Getting available numbers')
        if self.calling_enabled is False:
            log.debug("Not a Webex Calling Location")
            return None
        params = {'locationId': self.id, 'available': True}
        response = webex_api_call("get", "v1/telephony/config/numbers", params=params)
        if len(response.get('phoneNumbers', 0)) == 0:
            return None
        else:
            return response['phoneNumbers']

    @property
    def main_number(self):
        """Returns the Main Number for this Location, if defined

        Returns:
            str: The main number for the Location. None is returned if a main number cannot be found.

        """
        for number in self._parent.numbers:
            if number['location'].name == self.name and number['mainNumber'] is True:
                return number['number']
        return None

    @property
    def schedules(self):
        """ List of all :class:`wxcadm.LocationSchedule` instances for this Location"""
        log.debug('Getting Location Schedules')
        if self.calling_enabled is False:
            log.debug("Not a Webex Calling Location")
            return None
        response = []
        api_resp = webex_api_call("get", f"v1/telephony/config/locations/{self.id}/schedules", headers=self._headers)
        for schedule in api_resp['schedules']:
            log.debug(f'\tSchedule: {schedule}')
            this_schedule = LocationSchedule(self, schedule['id'], schedule['name'], schedule['type'])
            response.append(this_schedule)
        return response

    def set_announcement_language(self, language: str,
                                  update_users: bool = False,
                                  update_features: bool = False) -> bool:
        """ Set the Announcement Language for the Location or update the Announcement Language for existing users
        or features.

        When using this method, setting either ``update_users`` or ``update_features`` will only change the
        Announcement Language for existing users/features. It does not change the Location default that will be used
        for any new users or features. To ensure that the default language is changed, the method should be called
        without those arguments.

        Args:
            language (str): The language code (e.g. ``en_US``) to assign
            update_users (bool, optional): True to update all existing Users and Workspaces to the new language
            update_features (bool, optional): True to update all existing Features to the new language.

        Returns:
            bool: True on success, False otherwise

        """
        log.info(f"Setting language for {self.name} to {language}")
        payload = {
            'announcementLanguageCode': language,
            'agentEnabled': update_users,
            'serviceEnabled': update_features
        }
        success = webex_api_call('post',
                                 f'/v1/telephony/config/locations/{self.id}/actions/modifyAnnouncementLanguage/invoke',
                                 payload=payload)
        log.debug(f"Response: {success}")
        if success:
            log.info("Language change succeeded")
            return True
        else:
            log.info("Language change failed")
            return False

    def upload_moh_file(self, filename: str):
        """ Upload and activate a custom Music On Hold audio file.

        The audio file must be a WAV file that conforms with the Webex Calling requirements. It is recommended to test
        any WAV file by manually uploading it to a Location in Control Hub. The method will return False if the WAV file
        is rejected by Webex.

        Args:
            filename (str): The filename, including path, to the WAV file to upload.

        Returns:
            bool: True on success, False otherwise

        .. warning::

            This method requires the CP-API access scope.

        """
        log.debug(f'Uploading MOH file: {filename}')
        if self.calling_enabled is False:
            log.debug("Not a Webex Calling Location")
            return None
        upload = self._parent._cpapi.upload_moh_file(self.id, filename)
        if upload is True:
            log.debug('\tActivating MOH')
            activate = self._parent._cpapi.set_custom_moh(self.id, filename)
            if activate is True:
                log.debug('\t\tSuccessful activation')
                return True
            else:
                log.debug('\t\tActivation failed')
                return False
        else:
            return False

    def set_default_moh(self):
        """ Set the MOH to be the Webex Calling system default music.

        Returns:
            bool: True on success, False otherwise

        .. warning::

            This method requires the CP-API access scope.
        """
        log.debug('Setting Default MOH')
        if self.calling_enabled is False:
            log.debug("Not a Webex Calling Location")
            return None
        activate = self._parent._cpapi.set_default_moh(self.id)
        if activate is True:
            log.debug('\tSuccessful activation')
            return True
        else:
            log.debug('\tActivation failed')
            return False

    @property
    def outgoing_call_permissions(self):
        """ The Outgoing Call Permissions dicts (as a list) for the Location"""
        if self.calling_enabled is False:
            log.debug("Not a Webex Calling Location")
            return None
        ocp = webex_api_call('get', f'/v1/telephony/config/locations/{self.id}/outgoingPermission')
        return ocp['callingPermissions']

    def set_outgoing_call_permissions(self, outgoing_call_permissions: list) -> Optional[bool]:
        """ Ste the Outgoing Call Permissions for the Location

        This method uses the `callingPermissions` list style of the Webex API, which is the same format as returned by
        the :py:meth:`outgoing_call_permissions` property. The easiest method to change the permissions is to pull the
        `outgoing_call_permissions` list, modify it and pass it back to this method.

        Args:
            outgoing_call_permissions (list): The Webex `callingPermissions` list

        Returns:
            bool: True on success, False otherwise

        """
        log.info(f'Setting Outgoing Call Permission for Location: {self.name}')
        if self.calling_enabled is False:
            log.debug("Not a Webex Calling Location")
            return None
        log.debug(f'\tNew Permissions: {outgoing_call_permissions}')
        if not isinstance(outgoing_call_permissions, list):
            log.warning('outgoing_call_permissions is not a list')
            raise ValueError('outgoing_call_permissions must be a list')

        payload = {'callingPermissions': outgoing_call_permissions}

        success = webex_api_call('put', f'/v1/telephony/config/locations/{self.id}/outgoingPermission',
                                 payload=payload)
        if success:
            return True
        else:
            return False

    @property
    def park_extensions(self):
        """ List of :py:class:`CallParkExtension` instances for this Location """
        log.info(f"Getting Park Extensions for Location: {self.name}")
        if self.calling_enabled is False:
            log.debug("Not a Webex Calling Location")
            return None
        park_extensions = []
        response = webex_api_call("get", "v1/telephony/config/callParkExtensions", params={'locationId': self.id})
        log.debug(f"Response:\n\t{response}")
        for entry in response['callParkExtensions']:
            this_instance = CallParkExtension(self, entry['id'], entry['name'], entry['extension'])
            park_extensions.append(this_instance)

        return park_extensions

    def create_park_extension(self, name: str, extension: str):
        """ Create a new Call Park Extension at the Location

        Args:
            name (str): The name of the Park Extension
            extension (str): The Park Extension

        Returns:
            str: The ID of the newly created CallParkExtension

        """
        log.info(f"Creating Park Extension {name} ({extension}) at Location: {self.name}")
        if self.calling_enabled is False:
            log.debug("Not a Webex Calling Location")
            return None
        payload = {"name": name, "extension": extension}
        response = webex_api_call("post", f"v1/telephony/config/locations/{self.id}/callParkExtensions",
                                  params={'orgId': self._parent.id}, payload=payload)
        log.debug(f"Response:\n\t{response}")
        return response['id']

    @property
    def pickup_groups(self) -> Optional[PickupGroupList]:
        """ :py:class:`PickupGroupList` list of :py:class:`PickupGroup`s for this Location """
        log.info(f"Getting Pickup Groups for Location {self.name}")
        if self.calling_enabled is False:
            log.debug("Not a Webex Calling Location")
            return None
        if self._pickup_groups is None:
            self._pickup_groups = PickupGroupList(self)
        return self._pickup_groups



from __future__ import annotations

from typing import Union, Optional
from dataclasses import dataclass, field
from collections import UserList

import wxcadm.org
from wxcadm import log
from .common import *
from .location import Location

# This file is a placeholder for future development




from __future__ import annotations

from typing import Optional
import json

import wxcadm.location
from wxcadm import log
from .common import *


class ComplianceAnnouncementSettings:
    def __init__(self, parent: wxcadm.Org | wxcadm.Location,
                 inboundPSTNCallsEnabled: bool,
                 outboundPSTNCallsEnabled: bool,
                 outboundPSTNCallsDelayEnabled: bool,
                 delayInSeconds: int,
                 useOrgSettingsEnabled: Optional[bool] = None):
        self.parent: wxcadm.Org | wxcadm.Location = parent
        """ The :class:`Org` or :class:`Location` that the settings apply to """
        self.inbound_pstn_calls_enabled: bool = inboundPSTNCallsEnabled
        """ Play compliance announcement for inbound PSTN calls """
        self.outbound_pstn_calls_enabled: bool = outboundPSTNCallsEnabled
        """ Play compliance announcement for outbound PSTN calls """
        self.outbound_pstn_calls_delay_enabled: bool = outboundPSTNCallsDelayEnabled
        """ Delay the compliance announcement for a number of seconds for outbound PSTN calls """
        self.delay: int = delayInSeconds
        """ The number of seconds to delay the message for outbound calls, if enabled """
        self.use_org_settings: Optional[bool] = useOrgSettingsEnabled
        """ For Location-level Compliance Announcements, whether to override the Org settings (False) """

    def to_webex(self) -> dict:
        """ Represent the instance as a dict with the Webex field names as keys

        Returns:
            dict: The dict of Webex-friendly field names::

                {
                    'inboundPSTNCallsEnabled': self.inbound_pstn_calls_enabled,
                    'outboundPSTNCallsEnabled': self.outbound_pstn_calls_enabled,
                    'outboundPSTNCallsDelayEnabled': self.outbound_pstn_calls_delay_enabled,
                    'delayInSeconds': self.delay,
                    'useOrgSettingsEnabled': self.use_org_settings
                }

        """
        webex_dict = {
            'inboundPSTNCallsEnabled': self.inbound_pstn_calls_enabled,
            'outboundPSTNCallsEnabled': self.outbound_pstn_calls_enabled,
            'outboundPSTNCallsDelayEnabled': self.outbound_pstn_calls_delay_enabled,
            'delayInSeconds': self.delay,
            'useOrgSettingsEnabled': self.use_org_settings
        }
        return webex_dict

    def to_json(self) -> str:
        """ Represent the instance as a JSON string

        Returns:
            str: The JSON representation of the configuration

        """
        return json.dumps(self.to_webex())

    def push(self) -> bool:
        """ Push any changes (the existing instance config) back to Webex

        Returns:
            bool: True on success, False otherwise

        """
        if isinstance(self.parent, wxcadm.Org):
            pass
        pass

